/*==================================================================================
** COMPANY		= Wolverhampton University - Third Year Project
** PROJECT      = Temperature Data Recorder
** MODULE       =
** VERSION      = 0.0
** DATE         = 08-06-2010
** LAST CHANGE  =
** =================================================================================
** Description: Main Temperature Recording Programme
**
** Enviroment:  Device: PIC 16F684   	 	    Version:
**              Assembler: HI-TECH PICC Lite    Version: Version 9.60
**              C-Compiler: HI-TECH PICC Lite   Version: Version 9.60
**              Linker: HI-TECH PICC Lite       Version: Version 9.60
**				Programmer: PICkit(tm)1  	    Version: Flash Starter Kit, V1.60.0
**														 Firmware Version 2.0.2.
**
** =================================================================================
** By:          James Mason
** =================================================================================
Changes:
** =================================================================================
*/

#include	<htc.h>
#include 	<STDIO.h>
#include "Serial.h"
#include "DS18B20.h"

// GLOBAL Defines
#define _XTAL_FREQ 8000000
#define TRUE	0x01
#define FALSE	0x00
#define DQ	RC0

// Defines E2PROM Address
#define E2SERIAL_NUMBER			0x00
#define E2CONTROLLER_COUNTER	0x02

/* Configuration Bits */

// Data Memory Code Protect Disabled
// Brown Out Detect Enabled
// RA3 is a Digital Input
// Power Up Timer Enabled
// Watch Dog Disabled
// RA4/RA5 enabled as I/O

__CONFIG(INTIO & WDTDIS & PWRTEN & MCLRDIS & UNPROTECT & BORDIS & IESODIS & FCMDIS);



//Global Variables
unsigned long user_num;

unsigned int Count;
unsigned int Temperature;

unsigned char Address;
unsigned char SerialFlag;


void init(void)
{
	// Set the clock oscillator to the internal 8MHz
	// IRCF = 111 - 8Mhz
	// OSTS = 0 - Device running from internal system clock
	// SCS = 1 - Internal Oscillator is used for system clock
	OSCCON = 0x71;

	// Oscillator Module is running at the calibrated frequency
	OSCTUNE = 0x00;

	// This disables the comparator module 1 to allow digital I/O
	CMCON0 = 0x07;

	// This makes sure Capture/Compare/PWM are turned off
	// Not required with the 16F636
	//CCP1CON = 0X00;

	// This disables the A/D module to allow digital I/O
	// Not required with the 16F636
	//ANSEL=0;

	/* INTCON Interrupt Initialisation */
	INTE = 1;	// Enables RA2/INT External Interrupt
	RAIE = 0;	// Disables Port A Change Interrupt

	/* Timer 0 Initialisation */
	// Port A Pull-ups Disabled
	// Interrupt on Falling Edge of RA2
	// Internal Instruction Clock Cycle
	// Increment on High/Low Transition of RA2
	// Prescaller assigned to Timer 0
	// Prescaller Timer Rate 1:16

	OPTION = 0x93;

	/* Timer 1 Initialisation */
	// External Oscillator runnung at 32.768KHz
	// Timer Gate is not inverted (bit 7 = 0)
	// Timer Gate not enabled (bit 6 = 0)
	// Prescaller Timer Rate 1:4 (bit 5-4 = 10)
	// LP Oscillator enabled (bit 3 = 1)
	// Synchronise external clock input (bit 2 = 0)
	// External Clock from T1CKI pin - rising edge (bit 1 = 1)
	// Timer 1 - NOT STARTED (bit 0 = 0)

	T1CON = 0x29;

	// TMR1 to start at 0x8000 (32768)
	// to generate an overflow interrupt every 2 seconds
	TMR1H = 0x80;
	TMR1L = 0x00;

	T1CON |= 0x01;	// Start Timer 1

	/* WATCHDOG Timer Initialisation */

	WDTCON = 0x08;		// Prescaller Rate 1:512

	/* I/O Initialisation */

	// PORTA	Port A Register
	// TRISA	Port A Data Direction Register
	// WPUA		Weak Pull Up Resistor Register

	// PORTC	Port C Register
	// TRISC	Port C Tri State Data Direction Register

	// Inputs are set High (1)
	// Outputs are set Low (0)

	//
	//	|RA5|RA4|RA3|RA2|RA1|RA0|
	//	|RC5|RC4|RC3|RC2|RC1|RC0|
	//

	// RA0	=	Input	<<< RX 				RC0	=	Output	<<<
	// RA1	=	Output	<<< TX 				RC1 =	Output	<<<
	// RA2	=	Output	<<<					RC2	=	Output	<<<
	// RA3	=	Input	<<<					RC3	=	Input	<<<
	// RA4	=	Input	<<< Thermistor		RC4	=	Input	<<<
	// RA5 	=	Output	<<<					RC5 =	Input	<<<

	 TRISA = 0x19;
	 TRISC = 0x38;

	// Interrupts enabled after this point
	PEIE = 1;
	GIE = 1;	// Set the global interrupts
	INTCON = 0;		// Disable Interrupts

	DQ = TRUE;	// On power up set DQ high

	/* ==============================================================
	**
	**	The following section of code switches to serial comms
	**	if the serial comms is connected at power up
	**
	** ==============================================================
	*/

	if (RA0 == 0)
	{
		SerialFlag = TRUE;
	}
	else
	{
		SerialFlag = FALSE;
	}
}

void main (void)
{
 	unsigned char uart_data;

	init ();

	if (!SerialFlag)
	{
		while (1)
		{

		}
	}

	else
	{
		putch (0x0D);	// Carriage Return
       	putch (0x0A);	// Line Feed
		printf ("RS232 Comms Now Enabled\n\r(c)James Mason - Wolverhampton University\n\rThird Year Project - Temperature Data Logger\n\r");
		newline();

		while (1)
		{
			uart_data=getch();

		// If the data recieved is a HEX Value then convert that value to Decimal
		   if ((uart_data>='0' && uart_data<='9')||(uart_data>='A' && uart_data<='F'))
		   {
			  user_num *=16;              // left shift four places
			  uart_data = uart_data - '0';
			  if (uart_data > 9)
				  uart_data -= 7;
			  user_num += uart_data;      // convert uart data to binary

		   }
		   else
		   {
			   switch (uart_data)
			   {

					case 'J':
							putch ('A');
							break;

					case 'K':
							putch ('B');
							break;

					case 'L':
							putch ('C');
							break;

					default:
							newline();
				} 	 // end of switch

				newline();
		   }	// end of else
	   }	// end of while (1) loop
   }	// end of main else
}	// end of main






